# -*- coding: utf-8 -*-
#
# Tiedosto miinaharava.py
#
# Tämä ohjelma on tehty osaksi Aalto-yliopiston ohjelmoinnin oppimateriaaliprojektia.
#
# Tekijät: Teemu Koskinen ja Kimmo Kiiski 2010. 
#
# Jos ä- ja ö-merkit eivät näy oikein ja käytät Eclipseä, vaihda Project-valikon 
# Properties-kohdan kautta käytetty merkistökoodaus. Koodauksen voit vaihtaa kohdasta
# "Text file encoding". Oikea merkistö on UTF-8.
from miinaharava import *
from Tkinter import *
from tkMessageBox import *
import sys, os


class MiinapeliGUI(Tk):
    """ MiinapeliGUI-luokka perii Tkinter-käyttöliittymäkirjaston 
     Tk-luokan, joka vastaa käyttöliittymän ikkunoista.  
     Lisätietoa perinnästä: http://docs.python.org/tutorial/classes.html#inheritance """
     
    # Määritellään vakiot, joita kätetään vaikeustasoina.
    __HELPPO = 0
    __KESKIVAIKEA = 1
    __VAIKEA = 2


    def __init__(self):
        """ Tämä metodi alustaa miinapelin käyttöliittymäikkunan ja asettaa siihen
         myös muut käyttöliittymäkomponentit, kuten valikon ja tilarivin.
         Pelin käyttöliittymän luomisen jälkeen peli alkaa automaattisesti. """
        
        # Kutsutaan yliluokan __init__-metodia ja asetetaan pääikkunan 
        # otsikoksi "Miinaharava".
        Tk.__init__(self)
        self.title("Miinaharava")
        
        # Lisätään valikot tähän ikkunaan
        self.__lisaa_valikot()
        
        # Luodaan tilarivi
        self.__tilarivi = Tilarivi(self)
        
        # Asetetaan tilarivi ikkunan alalaitaan
        self.__tilarivi.grid(column=0, row=1, sticky=(W, E, N, S), padx=0, pady=0)
        
        # Aloitetaan peli ja käytetään vaikeustasona oletuksena keskivaikeaa
        self.__aloita_peli(self.__KESKIVAIKEA)


    def __lisaa_valikot(self):
        """ Tämä metodi lisää käyttöliittymäikkunaan valikot. """
        
        # Luodaan uusi valikko, joka toimii päävalikkona
        paavalikko = Menu(self)

        # Luodaan Peli-valikko
        pelivalikko = Menu(paavalikko, tearoff=0)
        
        # Lisätään peli-valikko päävalikkoon:
        paavalikko.add_cascade(label="Peli", menu=pelivalikko)
        
        # Luodaan valikko eri vaikeustasoja varten
        uudetpelit = Menu(pelivalikko, tearoff=0)

        # Lisätään uuden pelin aloittava uudetpelit-alivalikko Peli-valikkoon:
        pelivalikko.add_cascade(label="Uusi peli", menu=uudetpelit)
        
        # Lisätään kolme vaikeustasoa uudetpelit-valikkoon.
        # Pelin käynnistämiseksi halutulla vaikeustasolla määritellään uudet labda-funktiot.
        # Lisätietoa lambda-funktioista: http://docs.python.org/reference/expressions.html#lambda
        uudetpelit.add_command(label="Helppo",   command=lambda: self.__aloita_uusi_peli(self.__HELPPO))
        uudetpelit.add_command(label="Normaali", command=lambda: self.__aloita_uusi_peli(self.__KESKIVAIKEA))
        uudetpelit.add_command(label="Vaikea",   command=lambda: self.__aloita_uusi_peli(self.__VAIKEA))

        # Lisätään erotin pelivalikkoon:
        pelivalikko.add_separator()
        
        # Lisätään Lopeta-komento:
        pelivalikko.add_command(label="Lopeta", command=self.quit)

        # Luodaan Ohje-valikko ja lisätään se ikkunan päävalikkoon:
        ohjevalikko = Menu(paavalikko, tearoff=0)
        paavalikko.add_cascade(label="Ohje", menu=ohjevalikko)
        
        # Lisätään Ohje-valikkoon Peliohje ja Tietoa ohjelmasta -valinnat:
        ohjevalikko.add_command(label="Peliohje", command=lambda: OhjeIkkuna())
        ohjevalikko.add_separator()
        ohjevalikko.add_command(label="Tietoa ohjelmasta", command=self.nayta_tietoikkuna)
        
        # Asetetaan luotu päävalikko ikkunan yläreunaan:
        self.config(menu=paavalikko)


    def __luo_sektorit(self):
        
        # Luodaan sektorikehys, jonka sisälle laitetaan pelissä käytettävät sektorit.
        self.__sektorikehys = Frame(self, relief=SUNKEN, borderwidth=2, padx=5, pady=5)
        
        # Laitetaan ikkunan sisältö venymään sen koon mukaan sekä 
        # leveys-että pystysuunnassa painoarvoilla 1.
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=1)
        
        # Lisätään kehys pelialueelle grid-metodilla.
        self.__sektorikehys.grid(column=0, row=0, sticky=(W, E, N, S), padx=0, pady=0)
         
        # Luodaan sama määrä sektoreita kuin pelissä on ruutuja.
        # Lisätään kaikki sektorit kaksiulotteiseen listaan, jossa 
        # niiden indeksit vastaavat niiden koordinaatteja pelissä.
        self.__sektorit = []
        
        # Luodaan sarakkeiden verran listoja ulomman listan sisään.
        for x in range(self.__leveys):
            self.__sektorit.append([])
            
            # Määritellään kukin sarake venymään ikkunan kokoa muutettaessa
            # painotuksella 1.    
            self.__sektorikehys.columnconfigure(x, weight=1)
            
            # Luodaan jokaiseen sarakkeeseen korkeuden verran rivejä (eli Sektori-olioita).
            for y in range(self.__korkeus):
                ruutu = self.__peli.anna_ruutu(x, y)
                
                # Määritellään rivit venymään ikkunan kokoa muutettaessa painotuksella 1.
                # Tehdään tämä ainoastaan ensimmäisellä iteraatiokierroksella.
                if x == 0:
                    self.__sektorikehys.rowconfigure(y, weight=1)

                # Luodaan uusi sektori-olio, ja annetaan sille parametreina koordinaatit sekä 
                # viittaus tähän Miinapeli-olioon. 
                uusi_sektori = Sektori(self.__sektorikehys, ruutu, self)
                
                # Lisataan uusi sektori sektorilistaan
                self.__sektorit[x].append(uusi_sektori)
                
                # Asetetaan sektori omalle paikalleen käyttöliittymään, 
                # asetetaan se venymään jokaisesta reunastaan ja määritellään tyhjäksi tilaksi 0.
                uusi_sektori.grid(column=x, row=y, sticky=(W, E, N, S), padx=0, pady=0)


    def __aseta_vaikeustaso(self, vaikeustaso):
        """ Asettaa pelin vaikeustason, eli pelialueen koon ja siihen 
         sijoitettavien miinojen määrän. """
        if vaikeustaso == self.__HELPPO:
            self.__leveys = 9
            self.__korkeus = 9
            self.__miinoja = 10
        elif vaikeustaso == self.__KESKIVAIKEA:
            self.__leveys = 16
            self.__korkeus = 16
            self.__miinoja = 40
        else:
            self.__leveys = 30
            self.__korkeus = 16
            self.__miinoja = 99
        

    def __aloita_peli(self, vaikeustaso=None):
        """ Tämä metodi aloittaa uuden pelin. Se siis luo uuden Miinaharava-olion,
         ja piirtää käyttöliittymään uuden pelikentän. Vaikeustaso-parametriarvo on
         valinnainen, ja mikäli se jätetään antamatta, käytetään samaa vaikeustasoa
         kuin edellisessä pelissä.""" 

        # Jos vaikeustaso on annettu, asetetaan sitä vastaavat mitat ja miinojen määrä
        # __aseta_vaikeustaso-metodissa
        if vaikeustaso != None:
            self.__aseta_vaikeustaso(vaikeustaso)
        
        # Luodaan uusi Miinapeli
        self.__peli = Miinapeli(self.__leveys, self.__korkeus, self.__miinoja)
        
        # Kutsutaan sektorit alustavaa luo_sektorit-metodia
        self.__luo_sektorit()
        
        # Asetetaan tilariville aloitusteksti
        self.__tilarivi.aseta_teksti("Aloita klikkaamalla ruutua.")


    def kerro_viereisten_miinojen_maara(self, ruutu):
        """ Hakee peli-oliolta tiedon siitä, monta miinaa sijaitsee annettujen
         koordinaattien vieressä. """

        return self.__peli.miinoja_ruudun_vieressa(ruutu)


    def __aloita_uusi_peli(self, vaikeustaso=None):
        """ Aloittaa uuden pelin. Vaikeustaso-parametriarvo on valinnainen, ja mikäli se jätetään 
         antamatta, käytetään samaa vaikeustasoa kuin edellisessä pelissä. """
        
        # Poista vanhat sektorit sisältävä kehys, ja samalla kaikki sektorit
        self.__sektorikehys.destroy()
        
        # Aloita uusi peli
        self.__aloita_peli(vaikeustaso)
        
        
    def __hae_sektori(self, x, y):
        """ Palauttaa annetuissa koordinaateissa sijaitsevan sektorin. """
        return self.__sektorit[x][y]
    
    
    def __piirra_pelikentta(self):
        """ Päivittää pelikentän näkymän vastaamaan pelin tilannetta. """
        
        # Käydään läpi kaikki pelin sektorit ja käsketään 
        # niitä päivittämään näkymänsä
        for x in range(self.__leveys):
            for y in range(self.__korkeus):
                sektori = self.__hae_sektori(x, y)
                
                sektori.paivita_nakyma()
        
        # Päivitetään tilarivi
        self.__tilarivi.aseta_teksti("Siirtoja: " + str(self.__peli.kerro_siirrot()))
        

    def avaa_ruudun_naapurit(self, ruutu):
        """ Tätä metodia kutsutaan, kun kaikki parametrina annettavan ruudun 
         viereiset liputtamattomat ruudut halutaan avata kerralla. """
         
        self.__peli.avaa_viereiset_liputtomat(ruutu)
        self.__piirra_pelikentta()
        self.__tarkista_peli_kaynnissa()


    def sektori_klikattu(self, ruutu):
        """ Tämä metodi kutsuu Miinapeli-olion metodeita, joilla avataan parametriarvona
         annettu ruutu. Tätä metodia kutsutaan Sektori-olion klikkauksen jälkeen. """
        
        # Tarkastetaan onko peli käynnissä
        if self.__peli.onko_kaynnissa():
            # Käsketään Miinapeli-olion avata ruutu
            self.__peli.tee_siirto(ruutu)
            
            # Päivitetään käyttöliittymä
            self.__piirra_pelikentta()
            
            # Tarkastetaan onko peli vielä käynnissä
            self.__tarkista_peli_kaynnissa()
            
    
    def __tarkista_peli_kaynnissa(self):
        """ Tarkistaa onko peli päättynyt voittoon tai häviöön. Mikäli peli on päättynyt,
         näytetään asianmukainen ilmoitus pelaajalle. """
         
        # Jos peli ei ole käynnissä
        if not self.__peli.onko_kaynnissa():
            
            # Onko peli voitettu
            if self.__peli.onko_voittanut():
                # Pelaaja on voittanut, näytetään onnitteluteksti.
                viesti = "Onneksi olkoon, ratkaisit pelin! Haluatko pelata uudestaan?"
            else:
                # Pelaaja ei voittanut, ilmoitetaan tappiosta.
                viesti = "Voi ei, osuit miinaan. Haluatko pelata uudestaan?"
                
            # Kysytään, haluaako pelaaja aloittaa uuden pelin.
            if askyesno("Pelataanko uudestaan?", viesti):
                # Aloitetaan uusi peli.
                self.__aloita_uusi_peli()
            else:
                # Toivotetaan hassua päivänjatkoa :D
                showinfo(title="Hassua päivänjatkoa", message="Kiitos pelaamisesta ja hassua päivänjatkoa!")
                
                # Suljetaan ohjelma.
                self.quit()


    def nayta_tietoikkuna(self):
        """ Tämä metodi avaa tietoruudun jossa näytetään tietoa ohjelmasta. """
        
        tietoa  = "Python miinaharava v. 1.0.\n\n"
        tietoa += "Tämä peli on tehty Aalto-yliopiston ohjelmoinnin "
        tietoa += "oppimateriaaliprojektin screencast-esitysten tueksi.\n\n"
        tietoa += "Tekijät: Teemu Koskinen ja Kimmo Kiiski 2010. "
        
        showinfo(title="Tietoa ohjelmasta", message=tietoa)

class Sektori(Frame):
    """ Sektori-oliot kuvaavat yksittäisiä miinaharava-pelin käyttöliittymän ruutuja. 
     Sektori-luokka perii Tkinter-käyttöliittymäkirjaston Frame-luokan. """
     

    def __init__(self, ylakomponentti, ruutu, kayttoliittyma):
        """ Uudelle Sektorille annetaan parametreina viittaus siihen käyttöliittymä-
         komponenttiin, jonka sisälle se piirretään. Lisäksi se saa kokonaislukuina
         koordinaattinsa, sekä viittauksen siihen käyttöliittymäkomponenttiin, jonka 
         sisällä Sektori näytetään. """
         
        # Kutsutaan yliluokan __init__-metodia ja asetataan komponentin koko
        Frame.__init__(self, ylakomponentti, width=20, height=20)
        
        # Sijoitetaan ilmentymämuuttujaan viittaus MiinapeliGUI-olioon.
        # Tätä viittausta tarvitaan sektori_klikattu-metodin kutsumista varten.
        self.__kayttoliittyma = kayttoliittyma
              
        # Sijoitetaan tätä sektoria vastaava ruutu ilmentymämuuttujaan.
        self.__ruutu = ruutu
        
        # Asetetaan komponentti venymään ikkunan koon muuttamisen mukaan.
        self.grid_propagate(0)
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=1)
        
        # Kutsutaan __luo_painike-metodia, jossa luodaan uusi painike, eli Button-olio.
        self.__luo_painike()

        # Näillä muuttujilla pidetään kirjaa siitä, mikä sisältö painikkeessa näytetään.
        # Tiedon avulla voidaan välttää turhat painikkeen ulkoasun muutokset, kun 
        # käyttöliittymästä pyydetään päivittämään tämän sektorin näkymä.
        # Jos sektoreita päivitetään tarpeettomasti, ilmenee se käyttöliittymän vilkkumisena.
        self.__sektori_avattu = False
        self.__sektori_liputettu = False


    def __luo_painike(self):
        """ Tämä metodi luo uuden painikkeen, ja lisää sen tähän sektoriin. 
         Metodi huolehtii myös siitä, että painikkeen klikkaus hiiren eri napeilla saa 
         aikaan tarvittavan toiminnallisuuden. """
         
        # Luodaan uusi Button-olio
        self.__painike = Button(self, text=" ")

        # Asetetaan painikkeen ulkoasu näyttämään tavalliselta tyhjältä painikkeelta
        self.__painike.config(state=NORMAL, relief=RAISED, image=None, text=str(" "))
  
        # Asetetaan __painike_painettu-metodi suoritettavaksi painikkeen painalluksesta
        self.__painike.config(command=self.__painike_painettu)
      
        # Kiinnitetäään painikkeeseen hiiren keskipainikkeen toiminnallisuus
        self.__painike.bind('<Button-2>',  self.__avaa_viereiset) 
      
        # Kiinnitetään painikkeeseen hiiren kakkospainikkeen toiminnallisuus
        self.__painike.bind('<Button-3>',  self.__muuta_lippu) 
         
        # Lisätään painike käyttöliittymään tämän Sektorin sisään. 
        # Painike tulee venymään (sticky) kaikista suunnista ikkunan koon 
        # muuttuessa. Lisäksi sen tyhjäksi tilaksi määritellään 0.
        self.__painike.grid(sticky=(W, E, N, S), padx=0, pady=0)


    def __muuta_lippu(self, tapahtuma):
        """ Tämä metodi muuttaa liputtoman ruudun lipuksi ja lipullisen liputtomaksi, 
         jos ruutua ei ole vielä avattu. """
        
        # Lippu voidaan lisätä vain, jos ruutua ei ole avattu
        if not self.__ruutu.onko_auki():
            # Jos ruudussa on lippu, poistetaan se. Muussa tapauksessa lisätään lippu.
            if self.__ruutu.onko_lippu():
                self.__ruutu.poista_lippu()
            else: 
                self.__ruutu.aseta_lippu()
            
            # Päivitetään painikkeen ulkoasu
            self.paivita_nakyma()


    def __avaa_viereiset(self, tapahtuma):
        """ Tätä metodia kutsutaan käyttäjän klikatessa sektoria hiiren keskimmäisellä
         painikkeella. Metodi käskee käyttöliittymää avaamaan klikatun ruudun viereiset
         liputtamattomat ruudut. """
     
        self.__kayttoliittyma.avaa_ruudun_naapurit(self.__ruutu)


    def onko_lippu(self):
        """ Palauttaa tiedon siitä, onko tätä sektoria vastaava ruutu merkitty lipuksi. """
        return self.__rutuu.onko_lippu()
      

    def paivita_nakyma(self):
        """ Tämä metodi muuttaa sektorin avatuksi, eli näyttää sen käyttöliittymässä
         joko miinan kuvan tai vieressä olevien miinojen määrän. """

        if self.__ruutu.onko_auki():
            if self.__sektori_avattu:
                # Jos sektori on jo avoinna, ei sitä tarvitse turhaan muuttaa.
                # Tämä vähentää käyttöliittymän vilkkumista kun sektoreita 
                # ei päivitetä turhaan. Poistutaan metodista.
                return
            
            if self.__ruutu.onko_miina():
                # Jos ruudussa on miina, näytetään siinä miinan kuva. Jos kuvan lataus 
                # ei onnistu, näytetään ¤-merkki.
                self.__nayta_kuva("miina.gif", "¤")
                 
            # Jos ruutu ei ole miina:
            else: 
                # Oletuksena ruutua vastaavassa sektorissa 
                # näytetään vieressä olevien miinojen määrä.
                self.__nayta_miinat_vieressa()
                
            # Muutetaan sektori_avattu arvoksi True, jotta jatkossa ei päivitetä
            # painiketta turhaan. Tämä vähentää käyttöliittymän vilkkumista.
            self.__sektori_avattu = True


        elif self.__ruutu.onko_lippu():
            
            if self.__sektori_liputettu:
                # Jos sektori on jo liputettu, ei sitä tarvitse turhaan muuttaa.
                # Tämä vähentää käyttöliittymän vilkkumista kun sektoreita 
                # ei päivitetä turhaan. Poistutaan metodista.
                return
            
            # Yritetään käyttää kuvaa lippu-sektorissa. Jos kuvan lataus epäonnistuu,
            # näytetään sektorissa L-kirjain.    
            self.__nayta_kuva("lippu.gif", "L")
            
            # Muutetaan sektori_liputettu arvoksi True, jotta jatkossa ei päivitetä
            # painiketta turhaan.            
            self.__sektori_liputettu = True
            
        elif self.__sektori_liputettu:
            # Ruutua ei ole enää merkitty lipuksi, mutta painikkeessa on lipun kuva.
            # Poistetaan kuva painikkeesta luomalla kokonaan uusi painike.
            self.__painike.destroy()
            self.__luo_painike()
            self.__sektori_liputettu = False
        
        
    def __nayta_kuva(self, kuvatiedosto, merkki):
        """ Tällä metodilla voidaan näyttää GIF-kuva sektorin sisällä. Jos kuvan näyttäminen ei 
         onnistu, näytetään sektorissa parametriarvona annettu merkki. """
        try: 
            polku = os.path.join(sys.path[0], kuvatiedosto)
            self.__miina_kuva = PhotoImage(file=polku)
        
            # Poistetaan teksti ja korvataan se miinan kuvalla.
            self.__painike.config(image=self.__miina_kuva, text=None)
        except:
            # Jos kuvan näyttäminen epäonnistui, käytetään painikkeessa parametrina annettua merkkiä.
            self.__painike.config(text=merkki)
            
            
    def __nayta_miinat_vieressa(self):
        """ Tällä metodilla voidaan muuttaa sektorin näkymäksi vieressä olevien 
         miinoitettujen ruutujen lukumäärä. Metodia kutsutaan miinattoman sektorin avaamisen
         jälkeen, jos sen vieressä on miinoja. """


        # Selvitetään montako miinaa ruudun vieressä on.
        miinoja_vieressa = self.__kayttoliittyma.kerro_viereisten_miinojen_maara(self.__ruutu)
        
        # Avoimet ruudut näytetään taustan tasolla ja ne eivät ole klikattavissa.
        self.__painike.config(relief=FLAT, state=DISABLED)
        self.__painike.config(text=str(miinoja_vieressa))
        
        # Jos ruudulla on vieressä 3 tai enemmän miinoja, käytetään 
        # punaista fonttia, muussa tapauksessa sinistä.
        # Värit määritellään tässä heksadesimaalimuodossa.                        
        # Jos miinoja ei ole vieressä lainkaan, ei näytetä lainkaan lukuarvoa.
        if miinoja_vieressa >= 3:
            self.__painike.config(disabledforeground="#FF0000")
        elif miinoja_vieressa > 0: 
            self.__painike.config(disabledforeground="#0000FF")
        else:
            self.__painike.config(text=" ")   


    def __painike_painettu(self):
        """ Tätä metodia kutsutaan kun sektorin painiketta painetaan. 
         Metodi kutsuu edelleen Miinapeli-olion avaa_ruutu-metodia
         tämän sektorin koordinaateilla. 
         
         Jos sektori on merkitty lipuksi, ei painallus muuta sen 
         tilaa lainkaan. """
        if self.__ruutu.onko_lippu():
            return
        else:
            self.__kayttoliittyma.sektori_klikattu(self.__ruutu)



class OhjeIkkuna(Tk):
    """ Ohjeikkuna voidaan avata käyttöliittymän pääikkunan Ohje-valikosta. OhjeIkkuna-luokka
     perii käyttöliittymäkirjaston Tk-luokan, jonka avulla luodaan ikkunoita. """
    
    def __init__(self):
        """ Alustaa uuden OhjeIkkuna-olion ja kutsuu ohjetekstin lataavaa metodia. """
        Tk.__init__(self,)
        
        # Asetetaan ikkunan otsikoksi "Ohje" 
        self.title("Ohje")
                
        # Ladataan ohjeteksti
        self.__lataa_teksti()
        
        self.__nayta_ikkuna()

                
    def __lataa_teksti(self):
        """ Tämä metodi lataa pelin ohjeet tekstitiedostosta. """
                
        # Määritellään ohjeen sijainti ja yritetään avata se.
        polku = os.path.join(sys.path[0], "ohje.txt")
        try:
            tiedosto = open(polku)
            
            # Luetaan tiedoston sisältö ja viitataan siihen __ohjeteksti-ilmentymämuuttujalla.
            self.__ohjeteksti = tiedosto.read()
            
        except:
            self.__ohjeteksti = "Ohjetiedoston " + polku + " avaaminen ei onnistunut."
        
        
    def __nayta_ikkuna(self):
        """ Tämä metodi luo ja avaa ikkunan, jossa ohje näytetään. """
        
        # Luodaan vierityspalkki tekstikenttää varten.
        vierityspalkki = Scrollbar(self)
        
        # Luodaan uusi tekstikenttä, eli Text-olio. Text-luokka on osa käyttöliittymäkirjastoa.
        tekstikentta = Text(self, wrap=WORD, padx=5, pady=5, background="#ffffff")
        
        # Lisätään ohje tekstikenttään
        tekstikentta.insert(END, self.__ohjeteksti)
        
        # Poistetaan kirjoitusmahdollisuus tekstikentästä
        tekstikentta.config(state=DISABLED)
        
        # Lisätään tekstikenttä ikkunaan
        tekstikentta.grid(column=0, row=0, sticky=(W, E, N, S), padx=0, pady=0)
        
        # Asetetaan vierityspalkki toimimaan tekstikentän kanssa
        # ja lisätään se ikkunaan
        vierityspalkki.config(command=tekstikentta.yview)
        tekstikentta.config(yscrollcommand=vierityspalkki.set)
        vierityspalkki.grid(column=1, row=0, sticky=(W, E, N, S), padx=0, pady=0)
        
        # Asetetaan ikkunan asetukset niin, että tekstikenttä venyy ikkunan koon mukana, mutta 
        # vierityspalkin leveys pysyy vakiona
        self.columnconfigure(0, weight=1)
        self.columnconfigure(1, weight=0)
        self.rowconfigure(0, weight=1)
        
        
class Tilarivi(Frame):
    """ Tilarivi perii Tkinter-komponenteista Frame-luokan, jonka sisään voidaan
     laittaa muita käyttöliittymäkomponentteja. """
     
    def __init__(self, ylikomponentti, teksti=""):
        """ Tämä luokka toimii ikkunan alalaidassa näkyvänä tilarivinä. """
        Frame.__init__(self, ylikomponentti)
        self.columnconfigure(0, weight=1)

        # Luodaan tekstikomponentti ja lisätään se tämän tilarivin sisään.
        self.__teksti = Label(self, bd=1, relief=SUNKEN, anchor=W, text=teksti)
        self.__teksti.grid(column=0, row=0, sticky=(W, E, N, S), padx=0, pady=0)

        
    def aseta_teksti(self, teksti):
        """ Asettaa tilarivin tekstiksi parametrina annetun merkkijonon. """
        self.__teksti.config(text=teksti)
        self.__teksti.update_idletasks()


    
def main():
    # Luodaan käyttöliittymäikkuna ja käynnistetään käyttöliittymä
    peli = MiinapeliGUI()
    mainloop()

main()